# Drawing waveform and mass position

from PySide2 import QtWidgets, QtCore

from sohstationviewer.view.core import plotting_widget
from sohstationviewer.model.handling_data import trim_downsample_WFChan

from sohstationviewer.controller.plottingData import getTitle

from sohstationviewer.database import extractData

from sohstationviewer.conf.dbSettings import dbConf
from sohstationviewer.conf import constants as const

from sohstationviewer.controller.util import apply_convert_factor


class WaveformWidget(plotting_widget.PlottingWidget):
    """
    Widget to display waveform and mass position data.
    """
    def plot_channels(self, start_tm, end_tm, key,
                      data_time, channel_list, time_ticks_total,
                      waveform_data, mass_pos_data, gaps=[]):
        """
        Recursively plot each channels for waveform_data and mass_pos_data.

        :param start_tm: float - requested start time to read
        :param end_tm: float - requested end time to read
        :param key: str or (str, str) station name  for mseed,
            or (device's experiment number, experiment number ) for reftek
        :param data_time: [float, float] - time range of the selected data set
        :param channel_list: [str,] - waveform channel list read from the
            selected data set
        :param time_ticks_total: int - max number of tick to show on time bar
        :param waveform_data: dict - read waveform data of selected data set,
            refer to DataTypeModel.__init__.waveform_data[key]['read_data']
        :param mass_pos_data: dict - mass position data of the selected
            data set, refer to DataTypeModel.__init__.mass_pos_data[key]
        :param gaps: [[float, float],] - list of gaps of the selected data set.
            Gaps bar won't be displayed but is declared to be match with base
            method.
        """
        self.plotting_data1 = waveform_data
        self.plotting_data2 = mass_pos_data
        self.processing_log = []     # [(message, type)]
        self.gap_bar = None
        if self.axes:
            self.fig.clear()
        self.date_mode = self.parent.date_format.upper()
        self.time_ticks_total = time_ticks_total
        self.min_x = max(data_time[0], start_tm)
        self.max_x = min(data_time[1], end_tm)
        self.plot_total = len(self.plotting_data1) + len(self.plotting_data2)
        title = getTitle(key, self.min_x, self.max_x, self.date_mode)
        self.plotting_bot = const.BOTTOM
        self.plotting_bot_pixel = const.BOTTOM_PX
        self.axes = []

        self.timestamp_bar_top = self.add_timestamp_bar(0.003)
        self.set_title(title)

        for chan_id in self.plotting_data1:
            chan_db_info = extractData.getWFPlotInfo(chan_id)
            if chan_db_info['plotType'] == '':
                continue
            self.plotting_data1[chan_id]['chan_db_info'] = chan_db_info
            self.get_zoom_data(self.plotting_data1[chan_id], chan_id, True)
        for chan_id in self.plotting_data2:
            chan_db_info = extractData.getChanPlotInfo(
                chan_id, self.parent.data_type)
            self.plotting_data2[chan_id]['chan_db_info'] = chan_db_info
            self.get_zoom_data(self.plotting_data2[chan_id], chan_id, True)

        self.axes.append(self.plot_none())
        self.timestamp_bar_bottom = self.add_timestamp_bar(0.003, top=False)
        self.set_lim(first_time=True)
        self.bottom = self.axes[-1].get_ybound()[0]
        self.ruler = self.add_ruler(self.display_color['time_ruler'])
        self.zoom_marker1 = self.add_ruler(self.display_color['zoom_marker'])
        self.zoom_marker2 = self.add_ruler(self.display_color['zoom_marker'])
        # Set view size fit with the given data
        if self.main_widget.geometry().height() < self.plotting_bot_pixel:
            self.main_widget.setFixedHeight(self.plotting_bot_pixel)

        self.draw()

    def get_zoom_data(self, c_data, chan_id, first_time=False):
        """
        Trim data of a channel to the zooming time range and re-downsample
            the data to get more detail.

        :param c_data: dict - data of the channel which includes down-sampled
            data in keys 'times' and 'data'. Refer to
            DataTypeModel.__init__.waveform_data[key]['read_data'][chan_id] or
            DataTypeModel.__init__.mass_pos_data[key][chan_id]
        :param chan_id: str - name of channel
        :param plotting_data: dict - DataTypeModel.__init__.soh_data[key]. To
            help combine 2 channels in one axes in need
        :param first_time: bool - flag identify when the data set is first
            plotted, not zoom in yet
        """
        chan_db_info = c_data['chan_db_info']
        plot_type = chan_db_info['plotType']
        # data already processed for massposition in plottingWidget
        if not (chan_id.startswith('VM') or chan_id.startswith('MP')):
            trim_downsample_WFChan(
                c_data, self.min_x, self.max_x, first_time)
            apply_convert_factor(c_data, 1)
        # use ax_wf because with mass position, ax has been used
        # in plottingWidget
        if 'ax_wf' not in c_data:
            ax = getattr(self, dbConf['plotFunc'][plot_type][1])(
                c_data, chan_db_info, chan_id, None, None)
            if ax is None:
                return
            c_data['ax_wf'] = ax
            ax.chan = chan_id
            self.axes.append(ax)
        else:
            getattr(self, dbConf['plotFunc'][plot_type][1])(
                c_data, chan_db_info, chan_id, c_data['ax_wf'], None)


class WaveformDialog(QtWidgets.QWidget):
    def __init__(self, parent):
        super().__init__()
        """
        Dialog to display waveform and mass position data. Only pop up if
             'All' checkbox next to WFChans is checked or there are channels
             in WFChans textbox for mseed data or at least one of the DS
            checkbox is checked.

        :param parent: QMainWindow/QWidget - the parent widget
        """
        self.parent = parent
        """
        date_format: str - format to display date/time
        """
        self.date_format = self.parent.date_format
        """
        bit_weight_opt: str - option for bit weight ('', 'low', 'high')
            (Menu Options - Q330 Gain)
        """
        self.bit_weight_opt = self.parent.bit_weight_opt
        """
        mass_pos_volt_range_opt: str - ('regular'/'trilium'): define how to map
            values and colors when plotting
        """
        self.mass_pos_volt_range_opt = self.parent.mass_pos_volt_range_opt
        """
        data_type: str - type of data being plotted
        """
        self.data_type = None
        self.setGeometry(300, 300, 1200, 700)
        self.setWindowTitle("Raw Data Plot")

        main_layout = QtWidgets.QVBoxLayout()
        self.setLayout(main_layout)
        main_layout.setContentsMargins(5, 5, 5, 5)
        main_layout.setSpacing(0)

        """
        info_text_browser: QTextBrowser - to display info text
        """
        self.info_text_browser = QtWidgets.QTextBrowser(self)
        """
        plotting_widget: PlottingWidget - the widget to draw waveform and
            mass position channel
        """
        self.plotting_widget = WaveformWidget(
            self, self.info_text_browser, "waveformWidget")
        main_layout.addWidget(self.plotting_widget, 2)

        bottom_layout = QtWidgets.QHBoxLayout()
        main_layout.addLayout(bottom_layout)
        """
        save_button: save plot in plotting_widget to file
        """
        self.save_button = QtWidgets.QPushButton('Save', self)
        self.save_button.clicked.connect(self.save)
        bottom_layout.addWidget(self.save_button)
        self.info_text_browser.setFixedHeight(60)
        bottom_layout.addWidget(self.info_text_browser)

    def set_data(self, data_type, folder_name):
        """
        Set data_type and the window's title.

        :param data_type: str - data type of data being plotted
        :param folder_name: str - name of the folder of the data set to be
            displayed
        """
        self.data_type = data_type
        self.setWindowTitle("Raw Data Plot %s - %s" % (data_type, folder_name))

    def resizeEvent(self, event):
        """
        When WaveformDialog is resized, its plotting_widget need to initialize
            its size to fit the viewport.

        :param event: QResizeEvent - resize event
        """
        self.plotting_widget.init_size()

    @QtCore.Slot()
    def save(self):
        """
        Save the plotting to a file
        """
        print("save")
